use strict;
use warnings;
package Email::Sender::Manual;
BEGIN {
  $Email::Sender::Manual::VERSION = '0.110001';
}
# ABSTRACT: table of contents for the Email::Sender manual


1;

__END__
=pod

=head1 NAME

Email::Sender::Manual - table of contents for the Email::Sender manual

=head1 VERSION

version 0.110001

=head1 THE MANUAL

L<Email::Sender::Manual::QuickStart> tells you just what you need to know to
start using Email::Sender.

=head1 AUTHOR

Ricardo Signes <rjbs@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2011 by Ricardo Signes.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

